#!/usr/bin/env node
/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var path = require('path'),
  util = require('util'),
  misc = require('../lib/misc.js'),
  logger = require('../lib/logger.js'),
  procExitCode = misc.procExitCode,
  optimist = require('optimist')
    .alias('h', 'help')
    .describe('testdir', 'The test script folder')
    .describe('outdir', 'The output folder')
    .describe('packTime', 'Print the package build time stamp')
    .describe('updateTran', 'Auto-update transaction names to truapi.json')
    .usage('TruAPI command-line interface');

(function (argv) {
  var loggerOpts, testDir, outDir, engine, testBeginTM, localOpts;

  if (argv.help) {
    optimist.showHelp();
    process.exit(procExitCode.usage);
  }

  if (argv.packTime) {
    misc.printPackTime();
    process.exit(procExitCode.usage);
  }

  /* test settings */
  testBeginTM = Date.now();
  testDir = argv.testdir || argv.testDir || process.cwd();
  testDir = path.resolve(testDir);

  outDir = argv.outdir || argv.outDir || process.cwd();
  outDir = path.resolve(outDir);

  /* startup logger */
  function getDefaultLogFilename() {
    var filename;
    filename = util.format('truapi_%d_%d.log', process.pid, testBeginTM);
    return path.join(outDir, filename);
  }
  loggerOpts = {
    logLevel: argv.logLevel || 'debug',
    silent: argv.silent ? true : false,
    logFile: argv.logFile || getDefaultLogFilename(),
    outDir: outDir
  };
  logger.setOptions(loggerOpts);
  logger.debug('TruAPI CLI started');

  /* startup engine */
  localOpts = { testBeginTM: testBeginTM };
  engine = new require('../lib/engine_local.js').EngineLocal(localOpts);

  /* load script from folder */
  if (!engine.loadFromFolder(testDir, outDir)) {
    logger.error('Cannot load test. testDir = %s', testDir);
    process.exit(procExitCode.badTest);
  }

  /* run tests */
  engine.run(function (err) {
    var exitCode, now, tmSpan;
    exitCode = procExitCode.ok;
    now = Date.now();

    /* update truapi json */
    if (argv.updateTran) {
      logger.info('Update transaction names to %s', testDir);
      misc.tranNames.updateTruapiJSONFolder(testDir);
    }

    if (err) {
      logger.error('Test finished with error. Err = %s', err.toString());
      exitCode = procExitCode.badTest;
    }
    else {
      tmSpan = (now - testBeginTM) / 1000;
      tmSpan = (tmSpan < 0) ? 0 : tmSpan;
      logger.info('Test finished ( %d s )', tmSpan);
    }
    process.exit(exitCode);
  });
})(optimist.argv);

