/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var Engine = require('./engine.js').Engine,
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('engCloud'),
  fieldsName = misc.fieldsName,
  EngConnector = require('./cloud_connector.js').EngConnector,
  logger = require('./logger.js'),
  path = require('path'),
  _ = require('underscore'),
  util = require('util');

function EngineCloud(opts) {
  var self = this;
  if (!(this instanceof EngineCloud)) {
    return new EngineCloud(opts);
  }
  Engine.call(this);
  this._cloudOpts = opts || {};
  this._instanceParams = {
    enableForcefulPacing: false
  };

  /* startup cloud connector */
  this._msgEvt = {
    initVusers: function(evtData) {
      self.evtInitVusers(evtData);
    },
    run: function(evtData) {
      self.evtRun(evtData);
    },
    terminate: function(evtData) {
      self.evtTerminate(evtData);
    },
    exitInstance: function(evtData) {
      self.evtExitInstance(evtData);
    },
    vuserEvent: function(evtData) {
      self.evtVuserEvent(evtData);
    },
    stopTimerStage: function(evtData) {
      self.evtStopTimerStage(evtData);
    },
    instanceParams: function(evtData) {
      self.evtInstanceParams(evtData);
    }
  };
  this._connector = new EngConnector({
    cloudOpts: this._cloudOpts,
    onMsg: function(msg) {
      self.onCloudMsg(msg);
    }
  });
}

util.inherits(EngineCloud, Engine);
exports.EngineCloud = EngineCloud;

EngineCloud.prototype.onSubscribe = function(data) {
  var sndData;
  data = data || {};
  dbgTrace('subscribe evt, vu: %s', data.vuCtx.getVUserId());
  /* {
   *      eventName: subscribe,
   *      eventData: {
   *          identity: { vuserId: 's', testId: 's', groupId: 's' }
   *      }
   * }
   */
  sndData = {};
  sndData[fieldsName.identity] = data.vuCtx.fmtIdentity();
  this._connector.send('subscribe', sndData);
};

Engine.prototype.onVUserTerminated = function(data) {
  var sndData;
  data = data || {};
  dbgTrace('vuserTerminated evt, vu: %s', data.vuCtx.getVUserId());
  /* {
   *      eventName: vuserTerminated,
   *      eventData: {
   *          identity: { vuserId: 's', testId: 's', groupId: 's' }
   *      }
   * }
   */
  sndData = {};
  sndData[fieldsName.identity] = data.vuCtx.fmtIdentity();
  this._connector.send('vuserTerminated', sndData);
};

EngineCloud.prototype.onActionEnd = function(data) {
  var sndData, itrTotal, actCtx, stopped, vuAborted, suspendItr, self = this;

  data = data || {};
  actCtx = data.actCtx || {};

  itrTotal = _.isUndefined(actCtx.iterationTotal) ? 1 : actCtx.iterationTotal;
  itrTotal = Math.max(1, itrTotal);
  stopped = data.vuCtx.getData().stopTimerStage;
  suspendItr = data.vuCtx.getData().suspendIterations;
  vuAborted = data.vuCtx.getData().aborted;
  dbgTrace('actionEnd: act=%s, itrTotal=%d, itrCnt=%s, vuAborted=%s, vu=%s',
    data.action, itrTotal, data.itrCnt, vuAborted, data.vuCtx.getVUserId());
  dbgTrace('actionEnd: stopped = %s, suspendItr: = %s',
    stopped, suspendItr);

  if (!stopped && !vuAborted && data.itrCnt < itrTotal && !suspendItr) {
    /* call this action again */
    dbgTrace('run action again: act=%s, itrCnt=%d, total=%d, vu=%s',
       data.action, data.itrCnt, itrTotal, data.vuCtx.getVUserId());
    if (data.action === 'Action' && this._instanceParams.enableForcefulPacing) {
      dbgTrace('pacing action %s, interval %d, vu=%s',
        data.action, this._instanceParams.forcefulPacingInterval,
        data.vuCtx.getVUserId());
      setTimeout(function() {
        dbgTrace('run pacing action %s, interval %d, vu=%s',
           data.action, self._instanceParams.forcefulPacingInterval,
           data.vuCtx.getVUserId());
        self._runner.run({
          vuId: data.vuCtx.getVUserId(),
          action: data.action,
          actCtx: data.actCtx
        });
      }, this._instanceParams.forcefulPacingInterval);
    }
    else {
      setImmediate(function() {
        self._runner.run({
          vuId: data.vuCtx.getVUserId(),
          action: data.action,
          actCtx: data.actCtx
        });
      });
    }
  }
  else {
    /* {
     *      eventName: actionEnd,
     *      eventData: {
     *          identity: { vuserId: 's', testId: 's', groupId: 's' },
     *          action: 'Init|Action|End',
     *          iterationsDone: data.itrCnt
     *      }
     * }
     */
    sndData = {
      action: data.action,
      iterationsDone: data.itrCnt,
      aborted: vuAborted
    };
    sndData[fieldsName.identity] = data.vuCtx.fmtIdentity();
    dbgTrace('snd actionEnd: act=%s, itrCnt=%d, total=%d, vu=%s',
      data.action, data.itrCnt, itrTotal, data.vuCtx.getVUserId());
    this._connector.send('actionEnd', sndData);
  }
};

EngineCloud.prototype.onReport = function(rep) {
  rep = rep || {};
  dbgTrace('new report %s', rep.type);
  if (rep.type === 'message') {
    if (!rep.data || rep.data.msgType !== 0) {
      /* only send error message to result server */
      return;
    }
  }
  this._connector.send(rep.type, rep.data);
};

EngineCloud.prototype.onCloudMsg = function(msg) {
  var evtName, msgEvt;
  msg = msg || '';
  evtName = msg[fieldsName.evtName];
  dbgTrace('cloud new event %s', evtName);
  msgEvt = this._msgEvt[evtName];
  if (!msgEvt) {
    dbgTrace('Invalid message event %s', evtName);
    return;
  }
  msgEvt(msg[fieldsName.evtData]);
};

EngineCloud.prototype.loadFromFolder = function(dir) {
  var testCfgFile, cfg, test;
  dbgTrace('load test from %s', dir);
  testCfgFile = path.join(dir, 'truapi.json');
  testCfgFile = path.resolve(testCfgFile);
  cfg = misc.loadJSONSync(testCfgFile);
  if (!cfg || !cfg.test || !cfg.test.script) {
    logger.error('Cannot load test %s', testCfgFile);
    return false;
  }
  test = {
    script: path.resolve(dir, cfg.test.script),
    testId: this._cloudOpts.testId,
    groupId: this._cloudOpts.groupId
  };
  /* set runtimeconfig, cloud = true */
  if (!this.load(test)) {
    logger.error('Cannot load test script %s', test.script);
    return false;
  }
  return true;
};

EngineCloud.prototype.run = function(callback) {
  var self = this;
  dbgTrace('start run test');
  callback = callback || function() {};

  /* instance subscribe */
  function sndInstSubscribe() {
    self._connector.send('instanceSubscribe', self._cloudOpts.instanceId);
  }

  /* add 1 second delay for wait the cloud mediator servers ready */
  setTimeout(sndInstSubscribe, 1000);

  /* monitor test end event */
  this.once('testEnd', function() {
    dbgTrace('received cloud test end event');
    callback();
  });
};

EngineCloud.prototype.evtInitVusers = function(evtData) {
  var vuIds;
  vuIds = evtData || [];
  dbgTrace('init vusers, length %d', vuIds.length);

  function onVuCreated(data) {
    var vu;
    data = data || {};
    vu = data.vuser;
    if (vu) {
      /* init vusers context */
      vu._vuCtx.getData().stopTimerStage = false;
      vu._vuCtx.getData().suspendIterations = false;
    }
  }

  this.initVUsers(vuIds, {onVuCreated: onVuCreated});

  /* start subscribe */
  dbgTrace('start subscribe');
  this._runner.subscribe(vuIds);
};

EngineCloud.prototype.evtRun = function(evtData) {
  var vuId, actName, iterationTotal, actCtx;

  evtData = evtData || {};
  vuId = evtData[fieldsName.identity][fieldsName.vuserId];
  actName = evtData.action;
  iterationTotal = _.isUndefined(evtData.iterationTotal) ? 1 : evtData.iterationTotal;
  if (iterationTotal === -1) {
    iterationTotal = Infinity;
  }
  iterationTotal = Math.max(1, iterationTotal);
  dbgTrace('evt Run for %s (%s [%d])', vuId, actName, iterationTotal);
  actCtx = {iterationTotal: iterationTotal};
  this._runner.run({vuId: vuId, action: actName, actCtx: actCtx});
};

EngineCloud.prototype.evtVuserEvent = function(evtData) {
  /* TODO  support vuser event for rendezvous or other request */
  dbgTrace('evt vuserEvent: %j', evtData);
  evtData = evtData || {};
  if (evtData.message === 'suspendIterations') {
    this.usrEvtSuspendIterations(evtData);
  }
  else if (evtData.message === 'resumeIterations') {
    this.usrEvtResumeIterations(evtData);
  }
};

EngineCloud.prototype.evtTerminate = function(evtData) {
  var vuId;
  vuId = evtData[fieldsName.identity][fieldsName.vuserId];
  dbgTrace('evt terminate vu %s', vuId);
  this._runner.terminate([vuId]);
};

EngineCloud.prototype.evtExitInstance = function(evtData) {
  dbgTrace('evt exitInstance');
  /* TODO snd 'testEnd' event to main */
  process.exit(misc.procExitCode.ok);
};

EngineCloud.prototype.evtStopTimerStage = function(evtData) {
  var vuId, vu;

  vuId = evtData[fieldsName.identity][fieldsName.vuserId];
  dbgTrace('evt stopTimerStage vu %s', vuId);
  vu = this._runner.vuMgr.getVUser(vuId);
  if (!vu) {
    dbgTrace('Cannot find vuser %s', vu);
    return;
  }
  vu._vuCtx.getData().stopTimerStage = true;
};

EngineCloud.prototype.evtInstanceParams = function(evtData) {
  /* update instance parameters */
  evtData = evtData || {};
  dbgTrace('evtInstanceParams = %j', evtData);
  this._instanceParams.enableForcefulPacing = evtData.enableForcefulPacing;
  this._instanceParams.forcefulPacingInterval = evtData.forcefulPacingInterval;
  if (!_.isNumber(this._instanceParams.forcefulPacingInterval)) {
    this._instanceParams.forcefulPacingInterval = 0;
  }
};

EngineCloud.prototype.usrEvtSuspendIterations = function (evtData) {
  var vuId, vu;

  vuId = evtData[fieldsName.identity][fieldsName.vuserId];
  dbgTrace('evt suspendIterations vu %s', vuId);
  vu = this._runner.vuMgr.getVUser(vuId);
  if (!vu) {
    dbgTrace('Cannot find vuser %s', vu);
    return;
  }
  vu._vuCtx.getData().suspendIterations = true;
};

EngineCloud.prototype.usrEvtResumeIterations = function (evtData) {
  var vuId, vu;

  vuId = evtData[fieldsName.identity][fieldsName.vuserId];
  dbgTrace('evt resumeIterations vu %s', vuId);
  vu = this._runner.vuMgr.getVUser(vuId);
  if (!vu) {
    dbgTrace('Cannot find vuser %s', vu);
    return;
  }
  vu._vuCtx.getData().suspendIterations = false;
};

