/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var Engine = require('./engine.js').Engine,
  misc = require('./misc.js'),
  logger = require('./logger.js'),
  dbgTrace = misc.dbgTrace('engLocal'),
  util = require('util'),
  path = require('path'),
  reporter = require('./reporter.js');

function EngineLocal(opts) {
  if (!(this instanceof EngineLocal)) {
    return new EngineLocal(opts);
  }
  Engine.call(this);

  this._opts = opts || {};
  this._opts.testBeginTM = this._opts.testBeginTM || Date.now();

  /* local testing configuration */
  this._testCfg = {};
  this._subscribedCount = 0;
  this._mainActFinished = 0;
  this._endActFinished = 0;
  this._reporter = reporter.create();
  this._vuserCntMax = 500;
}

util.inherits(EngineLocal, Engine);
exports.EngineLocal = EngineLocal;

EngineLocal.prototype.onSubscribe = function (data) {
  data = data || {};
  dbgTrace('subscribe %j', data);
  if (!data.vuCtx) {
    logger.error('Invalid subscribe. Cannot find vuser');
    return;
  }
  this._subscribedCount++;
  dbgTrace('subscribed count %d', this._subscribedCount);
  logger.debug('vuser %s subscribed', data.vuCtx.getVUserId());
  if (this._subscribedCount >= this._testCfg.local.vuserCount) {
    dbgTrace('all vusers subscribed');
    logger.debug('vusers init');
    this._runner.runAllVusers({ action: 'Init' });
  }
};

EngineLocal.prototype.onActionEnd = function (data) {
  var vuCtx;

  data = data || {};
  dbgTrace('actEnd %j', data);
  vuCtx = data.vuCtx;
  if (!vuCtx) {
    logger.error('Invalid ActionEnd. Cannot find vuser');
    return;
  }
  logger.debug('vuser %s (%s) end', vuCtx.getVUserId(), data.action);
  if (vuCtx.getData().aborted && data.action !== 'End') {
    logger.warn('vuser is aborted (%s)', vuCtx.getVUserId());
    this.onVUserAborted(data);
    return;
  }

  if (data.action === 'Init') {
    this.onInitActEnd(data);
  }
  else if (data.action === 'Action') {
    this.onMainActEnd(data);
  }
  else if (data.action === 'End') {
    this.onEndActEnd(data);
  }
  else {
    logger.error('Invalid ActionEnd. Bad action name %s', data.action);
  }
};

EngineLocal.prototype.onVUserAborted = function (data) {
  this._mainActFinished++;
  if (this._mainActFinished >= this._testCfg.local.vuserCount) {
    dbgTrace('all vusers main action finished');
    this._runner.runAllVusers({ action: 'End' });
  }
};

EngineLocal.prototype.onInitActEnd = function (data) {
  var vuId;
  vuId = data.vuCtx.getVUserId();
  dbgTrace('init action end. Starting main action. vuId = %s', vuId);
  logger.debug('vuser %s, begin Action', vuId);
  this._runner.run({ vuId: vuId, action: 'Action' });
};

EngineLocal.prototype.onMainActEnd = function (data) {
  var vuId, itrCnt;
  vuId = data.vuCtx.getVUserId();
  itrCnt = data.itrCnt;
  dbgTrace('main action end. vu %s, itrCnt %d', vuId, itrCnt);
  logger.debug('vuser %s, begin Action=(%d)', vuId, itrCnt);
  /* check duration policy */
  if (itrCnt >= this._testCfg.local.policy.duration.param) {
    this._mainActFinished++;
  }
  else {
    dbgTrace('run next main action for vu %s', vuId);
    this._runner.run({ vuId: vuId, action: 'Action' });
  }
  if (this._mainActFinished >= this._testCfg.local.vuserCount) {
    dbgTrace('all vusers main action finished');
    this._runner.runAllVusers({ action: 'End' });
  }
};

EngineLocal.prototype.onEndActEnd = function (data) {
  var vuId, self = this;
  this._endActFinished++;
  vuId = data.vuCtx.getVUserId();
  dbgTrace('end action. vu %s.', vuId);

  function testEnd() {
    dbgTrace('send test end evt');
    self.emit('localTestEnd');
  }
  if (this._endActFinished >= this._testCfg.local.vuserCount) {
    dbgTrace('all end action finished');
    /*  add a delay for all the vuser ended */
    setTimeout(testEnd, 1000);
  }
};

EngineLocal.prototype.onReport = function (rep) {
  dbgTrace('new report %s', rep.type);
  this._reporter.addRcd(rep);
};

EngineLocal.prototype.loadFromFolder = function (dir, outDir) {
  var testCfgFile, cfg, test, duration;
  testCfgFile = path.join(dir, 'truapi.json');
  testCfgFile = path.resolve(testCfgFile);
  cfg = misc.loadJSONSync(testCfgFile);
  if (!cfg || !cfg.test || !cfg.test.script) {
    logger.error('Cannot load test %s', testCfgFile);
    return false;
  }
  test = {
    script: path.resolve(dir, cfg.test.script),
    testId: 'localtest',
    groupId: 'localGroup'
  };
  if (!this.load(test)) {
    logger.error('Cannot load test script %s', test.script);
    return false;
  }
  /* update local test option */
  this._testCfg = cfg;
  this._testCfg.local = cfg.local || {};
  this._testCfg.local.vuserCount = cfg.local.vuserCount || 1;
  if (0 >= this._testCfg.local.vuserCount) {
    this._testCfg.local.vuserCount = 1;
  }
  if (this._testCfg.local.vuserCount > this._vuserCntMax) {
    logger.warn('vuser count > max vuser count');
    this._testCfg.local.vuserCount = this._vuserCntMax;
  }

  /* check duration policy */
  this._testCfg.local.policy.duration = cfg.local.policy.duration || {};
  duration = this._testCfg.local.policy.duration;
  duration.type = 'iterationBased';
  duration.param = duration.param || 1;
  if (0 >= duration.param) {
    duration.param = 1;
  }

  /* create result CSV file */
  this._reporter.init({
    outDir: outDir,
    testBeginTM: this._opts.testBeginTM
  });
  dbgTrace('loaded test %j', this._testCfg);
  return true;
};

EngineLocal.prototype.run = function (callback) {
  var vuIds, localCfg, self = this;

  localCfg = this._testCfg.local;
  vuIds = misc.createRangeStrArray(localCfg.vuserCount);
  dbgTrace('Init vuser count = %d', localCfg.vuserCount);
  try {
    self.initVUsers(vuIds);
  }
  catch (err) {
    logger.error('Cannot initVUsers. vuserCount %d', localCfg.vuserCount);
    callback(new Error('Cannot initVUsers'));
    return;
  }

  /* subscribe */
  logger.debug('vusers subscribe');
  this._runner.subscribe(vuIds);

  /* monitor test end event */
  this.once('localTestEnd', function () {
    dbgTrace('received local test end event');
    self._reporter.close();
    callback();
  });
};

