/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var util = require('util'),
   events = require('events'),
   misc = require('./misc.js'),
   fieldsName = misc.fieldsName,
   dbgTrace = misc.dbgTrace('meter'),
   _ = require('underscore');

function Meter(ctx) {
   if (!(this instanceof Meter)) {
      return new Meter();
   }
   events.EventEmitter.call(this);
   this._ctx = ctx || {};
}

util.inherits(Meter, events.EventEmitter);
exports.Meter = Meter;

Meter.prototype._report = function(type, data) {
   dbgTrace('send report %s, data %j', type, data);
   this._ctx.onReport({type: type, data: data});
};

Meter.prototype.message = function(vuCtx, msg) {
   var rep;
   dbgTrace('new msg %s, %j', vuCtx.getVUserId(), msg);
   /* sample of message report:
    * {
    *  id: { "g": "1_testId0", "v": "0", "t": "testId0" },
    * "ts": 1397098150637,
    * "msgId": 0,
    * "msgType": 0,
    * "msg": "Action.c(43): Error: my error message"
    * }
    */
   rep = {
      ts: msg.ts,
      msgId: _.isNumber(msg.msgId) ? msg.msgId : 0,
      msgType: msg.msgType,
      msg: util.format('%s: %s', msg.level, msg.msg)
   };
   rep[fieldsName.identity] = vuCtx.fmtIdentity();
   this._report('message', rep);
};

Meter.prototype.transaction = function(vuCtx, tran) {
   var rep;
   dbgTrace('new tran %s, %j', vuCtx.getVUserId(), tran);
   /* sample of transaction report
    * {
    *    "id": { "g": "1_testId0", "v": "0", "t": "testId0" },
    *    "ts": 1397098134931,
    *    "wastedTime": 0,
    *    "startTime": 1397098134.930938,
    *    "endTime": 1397098134.931994,
    *    "name": "vuser_init_Transaction",
    *    "thinkTime": 0,
    *    "status": 0
    * }
    */
   rep = {
      ts: tran.ts,
      wastedTime: 0,
      startTime: tran.beginTS,
      endTime: tran.endTS,
      name: tran.name,
      thinkTime: tran.thinkTM
   };
   rep[fieldsName.identity] = vuCtx.fmtIdentity();
   rep.status = tran.statusCode;
   this._report('transaction', rep);
};

Meter.prototype.datapoint = function(vuCtx, dp) {
   var rep;
   dbgTrace('new dp %s, %j', vuCtx.getVUserId(), dp);
   /* example of datapoint report
    * {
    *   "id": { "g": "1_testId0", "v": "0", "t": "testId0" },
    *   "name": "tcp_connect",
    *   "type": 0,
    *   "value": 6,
    *   "ts": 1397098140539
    * }
    */
   rep = {
      ts: dp.ts,
      value: dp.value,
      type: dp.type,
      name: dp.name
   };
   rep[fieldsName.identity] = vuCtx.fmtIdentity();
   this._report('dataPoint', rep);
};

