/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var logger = require('./logger.js'),
   path = require('path'),
   util = require('util'),
   _ = require('underscore'),
   fs = require('fs');

exports.dbgTrace = require('debug');

exports.procExitCode = {
   ok: 0,
   badTest: 101,
   badTemplate: 201,
   usage: 300
};

exports.messageIDs = (function() {
   var minId;

   /* message IDs range
    * 0 ~ 50000 -  customer message ids
    * 50000+    -  truapi internal message ids
    */
   minId = 50001;
   return {
      minId: minId,
      testScriptErr: minId + 20
   };
})();

exports.loadJSONSync = function(filename) {
   var content, jsonObj;
   filename = path.resolve(filename);
   logger.debug('load json from %s', filename);
   try {
      content = fs.readFileSync(filename);
      jsonObj = JSON.parse(content);
   }
   catch (err) {
      logger.debug('Cannot load %s, err %s', filename, err.toString());
      return undefined;
   }
   return jsonObj;
};

exports.fieldsName = {
   evtName: 'n',
   evtData: 'd',
   identity: 'id',
   vuserId: 'v',
   testId: 't',
   groupId: 'g'
};

exports.createRangeStrArray = function(count) {
   var ret, idx;
   ret = [];
   for (idx = 0; idx < count; ++idx) {
      ret.push(util.format('%d', idx));
   }
   return ret;
};

exports.printPackTime = function() {
   var filename, data;

   filename = path.resolve(__dirname, '../.build_time');
   try {
      data = fs.readFileSync(filename);
      console.log('TruAPI Package build time stamp: %s', data.toString());
   }
   catch (err) {
      console.log('Cannot read build time stamp from %s.', filename);
   }
};

exports.tranNames = (function() {
   var allTranName, inst, maxTranCount;

   allTranName = {};
   maxTranCount = 200;

   function updateTruapiJSONObj(srcJSONObj) {
      var trans;

      srcJSONObj = srcJSONObj || {};

      /* keep the old transactions */
      trans = _.clone(allTranName);
      if (_.isArray(srcJSONObj.transactions)) {
         _.each(srcJSONObj.transactions, function(item) {
            trans[item] = true;
         });
      }
      srcJSONObj.transactions = _.keys(trans);
      /* check max length */
      if (srcJSONObj.transactions.length > maxTranCount) {
         srcJSONObj.transactions = srcJSONObj.transactions.slice(0, maxTranCount);
      }
      return srcJSONObj;
   }

   function updateTruapiJSONFile(srcJSONFile) {
      var srcContent, srcFilename, srcJSONObj;

      if (_.isEmpty(allTranName)) {
         /* nothing need update */
         return true;
      }

      /* load source json */
      srcFilename = path.resolve(srcJSONFile);
      try {
         srcContent = fs.readFileSync(srcJSONFile);
         srcJSONObj = JSON.parse(srcContent);
      }
      catch (err) {
         logger.warn('Cannot load %s, err %s', srcFilename, err.toString());
         return false;
      }

      /* update the json object */
      updateTruapiJSONObj(srcJSONObj);

      /* save this new json object */
      try {
         fs.writeFileSync(srcFilename, JSON.stringify(srcJSONObj, null, '   '),
            {flags: 'w'});
      }
      catch (err) {
         logger.warn('Cannot save %s, err %s', srcFilename, err.toString());
         return false;
      }
      return true;
   }

   inst = {
      addTran: function(name) {
         if (name) {
            allTranName[name] = true;
         }
         return inst;
      },
      getAllNames: function() {
         return allTranName;
      },
      updateTruapiJSONFile: updateTruapiJSONFile,
      updateTruapiJSONObj: updateTruapiJSONObj,
      clear: function() {
         allTranName = {};
      },
      getMaxUpdateCount: function() {
         return maxTranCount;
      },
      updateTruapiJSONFolder: function(testFolder) {
         return updateTruapiJSONFile(path.resolve(testFolder, 'truapi.json'));
      }
   };
   return inst;
})();

