/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var util = require('util'),
  events = require('events'),
  Meter = require('./meter.js').Meter,
  vuserManager = require('./vuser_manager.js'),
  runtimeCfg = require('./runtime_config.js'),
  vuser = require('./vuser.js'),
  logger = require('./logger.js'),
  misc = require('./misc.js'),
  dbgTrace = misc.dbgTrace('runner'),
  _ = require('underscore');

function Runner() {
  var meterCtx, self = this;
  if (!(this instanceof Runner)) {
    return new Runner();
  }
  events.EventEmitter.call(this);

  this.runtimeCfg = runtimeCfg.create();
  meterCtx = {
    onReport: function (data) {
      self.emit('report', data);
    }
  };
  this._meter = new Meter(meterCtx);
  this.vuMgr = vuserManager({
    runtimeCfg: this._runtimeCfg,
    meter: this._meter,
    runner: this
  });
  this._testId = '';
  this._groupId = '';
  this._testScript = function () {};
}

util.inherits(Runner, events.EventEmitter);
exports.Runner = Runner;

Runner.prototype.load = function (opts) {
  var script, mod;
  opts = opts || {};
  script = opts.script;
  try {
    /* load customer's script in Try...Catch block */
    dbgTrace('load script %s', script);
    mod = require(script);
  }
  catch (err) {
    logger.error('Cannot load script %s. err %s', script, err.toString());
    return false;
  }
  if (!_.isFunction(mod)) {
    logger.error('Cannot load script %s. Bad module', script);
    return false;
  }
  logger.debug('script %s loaded', script);
  this._testScript = mod;
  this._testId = opts.testId;
  this._groupId = opts.groupId;
  return true;
};

Runner.prototype.initVUsers = function (vusers, opts) {
  var self = this;
  vusers = vusers || [];
  opts = opts || {};

  dbgTrace('initVUsers, vusers: %j', vusers);
  opts.onVuCreated = opts.onVuCreated || function () {};
  _.each(vusers, function (vuId) {
    var vu;
    /* create new vuser object */
    vu = self.vuMgr.createNewVUser({
      groupId: self._groupId,
      testId: self._testId,
      vuserId: vuId
    });
    opts.onVuCreated({ vuser: vu });
    /* attach to script */
    try {
      self._testScript(vu);
    }
    catch (err) {
      logger.error('Cannot load vuser script. Err %s', err.toString());
      throw err;
    }
  });
};

Runner.prototype.run = function (opts) {
  var vu;
  opts = opts || {};
  if (!vuser.actName.isValidReqActName(opts.action)) {
    logger.error('Invalid action name %s', opts.action);
    return;
  }
  vu = this.vuMgr.getVUser(opts.vuId);
  if (!vu) {
    logger.error('Cannot find vuser %s', opts.vuId);
    return;
  }
  dbgTrace('start action %s, vu %s', opts.action, opts.vuId);
  vu.runAction(opts.action, opts.actCtx);
  return;
};

Runner.prototype.subscribe = function (vusers) {
  var self = this;
  _.each(vusers, function (vuId) {
    var vu;
    dbgTrace('subscribe, vu %s', vuId);
    vu = self.vuMgr.getVUser(vuId);
    vu.subscribe();
  });
};

Runner.prototype.runAllVusers = function (opts) {
  var allVUsers;
  opts = opts || {};
  if (!vuser.actName.isValidReqActName(opts.action)) {
    logger.error('Invalid action name %s', opts.action);
    return false;
  }
  allVUsers = this.vuMgr.getAllVUsers();
  _.each(_.values(allVUsers), function (vu) {
    dbgTrace('start action %s, vu %s', opts.action, vu.getVUserId());
    vu.runAction(opts.action);
  });
};

Runner.prototype.terminate = function (vusers) {
  var self = this;
  _.each(vusers, function (vuId) {
    var vu;
    dbgTrace('terminate, vu %s', vuId);
    vu = self.vuMgr.getVUser(vuId);
    vu.terminate();
  });
};

