/*
 * #© Copyright 2019 - Micro Focus or one of its affiliates
 * #
 * # The only warranties for products and services of Micro Focus and its affiliates and licensors (“Micro Focus”)
 * # are as may be set forth in the express warranty statements accompanying such products and services.
 * # Nothing herein should be construed as constituting an additional warranty.
 * # Micro Focus shall not be liable for technical or editorial errors or omissions contained herein.
 * # The information contained herein is subject to change without notice.
 *
 */

'use strict';

var util = require('util'),
   events = require('events'),
   logger = require('./logger.js'),
   misc = require('./misc.js'),
   dbgTrace = misc.dbgTrace('vuser'),
   vuserSVC = require('./vuser_services.js');

/* action name converter */
exports.actName = (function() {
   var reqActName;
   reqActName = {};
   reqActName.Init = 'init';
   reqActName.Action = 'action';
   reqActName.End = 'end';
   return {
      toInternal: function(name) {
         return reqActName[name];
      },
      isValidReqActName: function(name) {
         return name === 'Init' || name === 'Action' || name === 'End';
      }
   };
})();

function VUser(opts) {
   var self = this;
   if (!(this instanceof VUser)) {
      return new VUser(opts);
   }
   events.EventEmitter.call(this);

   /* startup VUser */
   opts = opts || {};
   this._vuCtx = opts.vuCtx;
   this._runner = opts.runner;
   this._subscribed = false;
   this._terminated = false;

   /* init vuser context */
   this._vuCtx.getData().aborted = false;

   /* vuser services */
   this._vuSVC = vuserSVC({
      vuCtx: opts.vuCtx,
      meter: opts.meter,
      cfg: opts.cfg
   });
   this.getVUserId = opts.vuCtx.getVUserId;
   /* vuser actions */
   this._acts = {
      init: {itrCnt: 0},
      action: {itrCnt: 0},
      end: {itrCnt: 0}
   };
   this.init = function(desc, act) {
      self._setAction('init', desc, act);
   };
   this.action = function(desc, act) {
      self._setAction('action', desc, act);
   };
   this.end = function(desc, act) {
      self._setAction('end', desc, act);
   };
}

util.inherits(VUser, events.EventEmitter);
exports.VUser = VUser;

VUser.prototype.runAction = function(actName, ctx) {
   var act, actItem, internalActName, self = this;
   if (this._terminated) {
      logger.error('Cannot run action %s, vuser is already terminated', actName);
      return;
   }
   internalActName = exports.actName.toInternal(actName);
   actItem = this._acts[internalActName];
   if (!actItem) {
      logger.error('Invalid action name %s', actName);
      return;
   }
   act = actItem.act;
   if (!act) {
      dbgTrace('empty action %s', actName);
      actItem.itrCnt++;
      self._runner.emit('actionEnd', {
         action: actName,
         vuCtx: self._vuCtx,
         itrCnt: actItem.itrCnt,
         actCtx: ctx || {}
      });
      return;
   }

   /* run customer's code */
   function invokeAct() {
      var actDone, errMsg;

      actDone = false;
      try {
         /* run customer's code in try..catch  */
         act(self._vuSVC, function() {
            if (actDone) {
               /* the caller invoked the actionDone twice */
               errMsg = util.format('done() was called multiple times: %s',
                  new Error().stack);
               self._vuSVC.logger.error(errMsg);
               return;
            }

            actDone = true;
            actItem.itrCnt++;
            self._runner.emit('actionEnd', {
               action: actName,
               vuCtx: self._vuCtx,
               itrCnt: actItem.itrCnt,
               actCtx: ctx || {}
            });
         });
      }
      catch (err) {
         self._vuSVC.logger.error('action (%s) error:%s', actName, err.toString());
         actItem.itrCnt++;
         self._runner.emit('actionEnd', {
            action: actName,
            vuCtx: self._vuCtx,
            itrCnt: actItem.itrCnt,
            actCtx: ctx || {}
         });
      }
   }

   setImmediate(invokeAct);
};

VUser.prototype.subscribe = function() {
   var self = this;
   if (self._subscribed) {
      logger.error('Cannot call subscribe twice');
      return;
   }
   function invokeSubscribe() {
      self._subscribed = true;
      self._runner.emit('subscribe', {vuCtx: self._vuCtx});
   }

   setImmediate(invokeSubscribe);
};

VUser.prototype.terminate = function() {
   var self = this;

   function invokeTerminate() {
      self._terminated = true;
      self._runner.emit('vuserTerminated', {vuCtx: self._vuCtx});
   }

   setImmediate(invokeTerminate);
};

VUser.prototype._setAction = function(name, desc, act) {
   if (this._acts[name]) {
      this._acts[name].desc = desc;
      this._acts[name].act = act;
   }
};

VUser.prototype.abort = function () {
   logger.warn('vuser (%s) is aborting', this.getVUserId());
   this._vuCtx.getData().aborted = true;
};

